<div id="tab_plugin_octoprint_nanny">
    {% if plugin_octoprint_nanny_printnanny_user != none %}
    <h3>🔐 Log into PrintNanny</h3>
    <p>Link your PrintNanny account to automatically monitor print jobs, upload backups, and keep your files in sync.</p>
    <a href="{{ plugin_octoprint_nanny_config.dash.base_url }}" class="btn btn-primary btn-large">Link PrintNanny Account</a>
    {% endif %}

    {% include 'octoprint_nanny_janus.jinja2' %}

</div>